"""
read VS30 file

IN: dlm VS30 file `LON LAT VS30` (see `test/testvs30.txt`)
OUT: `Point_vs30` array
"""
function read_vs30_file(filename::String)
    A = readdlm(filename)
    vs30_row_num = length(A[:, 1])
    B = Array{Point_vs30}(undef, 0)
    for i = 1:vs30_row_num
        B = push!(B, Point_vs30(A[i, 1], A[i, 2], A[i, 3]))
    end
    return B
end

"""
read PGA grid file

IN: dlm GRID file `LON LAT PGA` (see `test/SakhGMPE.txt`)
OUT: `Point_pga_out` array
"""
function read_pga_file(filename::String)
    A = readdlm(filename)
    row_num = length(A[:, 1])
    B = Array{Point_pga_out}(undef, 0)
    for i = 1:row_num
        B = push!(B, Point_pga_out(A[i, 1], A[i, 2], A[i, 3]))
    end
    return B
end

"""
read intensity measures file

IN: dlm intensity file `LON LAT INTENSITY IS_STATION` (see
`test/felt_reports.txt` example)

OUT: `Point_felt_report` array
"""
function read_intensity_file(filename::String)
    A = readdlm(filename)
    intensity_row_num = length(A[:, 1])
    B = Array{Point_felt_report}(undef, 0)
    for i = 1:intensity_row_num
        B = push!(B, Point_felt_report(A[i, 1], A[i, 2], A[i, 3], A[i, 4]))
    end
    return B
end


"""
convert Array{Float64,2} array to Array{GroundMotion.Point_vs30,1}
"""
function convert_to_point_vs30(A::Array{Float64,2})
    vs30_row_num = length(A[:, 1])
    B = Array{Point_vs30}(undef, 0)
    for i = 1:vs30_row_num
        B = push!(B, Point_vs30(A[i, 1], A[i, 2], A[i, 3]))
    end
    return B
end

"""
convert Point_pga_out to Point_ssi_out (%g.gg to SSI intensity)

Please NOTE this function makes it possible to result different length of array
in case of filter out negative and zero SSI values.

"""
function convert_from_pga_to_ssi(
    A::Array{Point_pga_out};
    alpha=2.5,
    beta=1.89
)::Array{Point_ssi_out}
    g_global = 9.81
    B = Array{Point_ssi_out}(undef, 0)
    for i in A
        if i.pga > 0 # process only non-zero values
            pga_sm = i.pga * g_global # convert %g to sm/sec^2
            intensity = log10(pga_sm) * alpha + beta # convert to SSI
            if intensity > 0 # filter out negative intensity
                intensity = round(intensity, digits=2)
                push!(B, Point_ssi_out(i.lon, i.lat, intensity))
            end
        end
    end
    return B
end
function convert_from_pga_to_ssi(
    A::Array{Point_felt_report};
    alpha=2.5,
    beta=1.89
)::Array{Point_felt_report}
    g_global = 9.81
    B = Array{Point_felt_report}(undef, 0)
    for i in A
        if i.intensity > 0 # process only non-zero values
            pga_sm = i.intensity * g_global # convert %g to sm/sec^2
            intensity = log10(pga_sm) * alpha + beta # convert to SSI
            if intensity > 0 # filter out negative intensity
                intensity = round(intensity, digits=2)
                push!(
                    B,
                    Point_felt_report(i.lon, i.lat, intensity, i.is_station)
                )
            end
        end
    end
    return B
end

"""
convert Point_ssi_out to Point_pga_out (SSI intensity to %g.gg)
"""
function convert_from_ssi_to_pga(
    A::Array{Point_ssi_out};
    alpha=2.5,
    beta=1.89
)::Array{Point_pga_out}
    g_global = 9.81
    B = Array{Point_pga_out}(undef, 0)
    for i in A
        pga_g = 10^((i.ssi - beta) / alpha) / g_global
        pga_g = round(pga_g, digits=2)
        push!(B, Point_pga_out(i.lon, i.lat, pga_g))
    end
    return B
end
function convert_from_ssi_to_pga(
    A::Array{Point_felt_report};
    alpha=2.5,
    beta=1.89
)::Array{Point_felt_report}
    g_global = 9.81
    B = Array{Point_felt_report}(undef, 0)
    for i in A
        pga_g = 10^((i.intensity - beta) / alpha) / g_global
        pga_g = round(pga_g, digits=2)
        push!(B, Point_felt_report(i.lon, i.lat, pga_g, i.is_station))
    end
    return B
end

"""
read VS30 file

IN: dlm VS30 file LON LAT VS30 DL
OUT: Point_vs30_dl array
"""
function read_vs30_dl_file(filename::String)
    A = readdlm(filename)
    vs30_row_num = length(A[:, 1])
    B = Array{Point_vs30_dl}(undef, 0)
    for i = 1:vs30_row_num
        B = push!(B, Point_vs30_dl(A[i, 1], A[i, 2], A[i, 3], A[i, 4]))
    end
    return B
end

"""
convert_to_float_array(B::Array{T,N})
  where T is Point_{pga,pgv,pgd}_out, Point_vs30

Convert 1-d array of custom type to Array{Float64}(N,X)

Example: Array{Point_pga_out}(N,1) -> Array{Float64}(N,4)
"""
function convert_to_float_array(B::Array{Point_pga_out})
    A = Array{Float64}(undef, length(B), 3)
    for i = 1:length(B)
        A[i, 1] = B[i].lon
        A[i, 2] = B[i].lat
        A[i, 3] = B[i].pga
    end
    return A
end
function convert_to_float_array(B::Array{Point_ssi_out})
    A = Array{Float64}(undef, length(B), 3)
    for i = 1:length(B)
        A[i, 1] = B[i].lon
        A[i, 2] = B[i].lat
        A[i, 3] = B[i].ssi
    end
    return A
end
function convert_to_float_array(B::Array{Point_vs30})
    A = Array{Float64}(undef, length(B), 3)
    for i = 1:length(B)
        A[i, 1] = B[i].lon
        A[i, 2] = B[i].lat
        A[i, 3] = B[i].vs30
    end
    return A
end
function convert_to_float_array(B::Array{Point_vs30_dl})
    A = Array{Float64}(undef, length(B), 4)
    for i = 1:length(B)
        A[i, 1] = B[i].lon
        A[i, 2] = B[i].lat
        A[i, 3] = B[i].vs30
        A[i, 4] = B[i].dl
    end
    return A
end
